/*
 * compass.c
 *
 *  Created on: 2013/12/28
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <errno.h>
#include <pthread.h>
#include <sys/time.h>
#include <string.h>
#include <math.h>
#include "debug.h"
#include "i2c.h"
#include "compass.h"

/*---- コンパス構造体 ----*/
struct Compass_ {
	I2CDevice* device;             /* I2Cデバイス */
	uint16_t angle;                /* 計測結果 */
	struct timeval lastUpdateTime; /* 最終更新時刻 */
	pthread_mutex_t mutex;         /* 同期用mutex */
};

/*---- コマンドリスト ----*/
/* モード設定コマンド */
#define COMMAND_WRITE_TO_RAM      'G'
/* 校正開始コマンド */
#define COMMAND_START_CALIBRATION 'C'
/* 校正終了コマンド */
#define COMMAND_STOP_CALIBRATION  'E'
/* データ取得コマンド */
#define COMMAND_GET_DATA          'A'

/*---- 計測エラー値 ----*/
/* 未計測 */
#define COMPASS_RESULT_NO_DATA 0x8000

/*
 * 計測を開始する
 * 引数: コンパスポインタ
 * 戻値: 正常時 0
 *       異常時 -1
 */
static int compassStartMeasurement(Compass* compass){
	uint8_t command[1];
	int result;
	/* 引数確認 */
	if(compass == NULL){
		ERROR_PRINT("Compass is NULL\n");
		return -1;
	}
	/* コンパスロック */
	pthread_mutex_lock(&compass->mutex);
	/* 計測開始 */
	command[0] = COMMAND_GET_DATA;
	result = i2cWrite(compass->device,command,sizeof(command));
	if(result < (int)sizeof(command)){
		ERROR_PRINT("i2cWrite: failure\n");
		result = -1;
	}else{
		result = 0;
	}
	/* コンパスアンロック */
	pthread_mutex_unlock(&compass->mutex);
	return result;
}

/*
 * コンパスとの通信を開始する
 * 引数: I2Cバス,アドレス
 * 戻値: 正常時 コンパスポインタ
 *       異常時 NULL (errnoで判断)
 */
Compass* compassOpen(I2CBus* bus,uint8_t address){
	Compass* compass = NULL;
	int error;

	do{
		/* メモリ確保 */
		compass = (Compass*)malloc(sizeof(Compass));
		if(compass == NULL){
			error = errno;
			ERROR_PRINT("malloc: %s\n",strerror(errno));
			break;
		}
		/* デバイスオープン */
		compass->device = i2cDeviceOpen(bus,address);
		if(compass->device == NULL){
			error = errno;
			ERROR_PRINT("i2cDeviceOpen: failure\n");
			break;
		}
		/* メンバ初期化 */
		compass->angle = COMPASS_RESULT_NO_DATA;
		compass->lastUpdateTime.tv_sec = 0;
		pthread_mutex_init(&compass->mutex,NULL);
		return compass;
	}while(0);

	/* エラー時の処理 */
	if(compass != NULL){
		i2cDeviceClose(compass->device);
		free(compass);
	}
	errno = error;
	return NULL;
}

/*
 * コンパスとの通信を終了する
 * 引数: コンパスポインタ
 */
void compassClose(Compass* compass){
	/* 引数確認 */
	if(compass == NULL){
		return;
	}
	/* コンパスロック */
	pthread_mutex_lock(&compass->mutex);
	/* メモリ解放 */
	free(compass);
	return;
}

/*
 * 現在の方位を取得する
 * 引数: コンパスポインタ
 * 戻値: 正常時 測定結果 0 ~ 360 [deg]
 *       異常時 NaN
 */
double compassGetAngle(Compass* compass){
	uint8_t response[2];
	struct timeval time,timeSub;
	int result;
	double angle;
	/* 引数確認 */
	if(compass == NULL){
		return nan("");
	}

	/* コンパスロック */
	pthread_mutex_lock(&compass->mutex);

	/* 前回の更新からの時間を計測 */
	gettimeofday(&time, NULL);
	timersub(&time, &compass->lastUpdateTime, &timeSub);

	/* 更新時間間隔を超えていた場合は更新する */
	if((timeSub.tv_sec + timeSub.tv_usec * 1e-6) >= (COMPASS_INTERVAL_TIME * 1e-6)){
		/* 計測命令送信 */
		pthread_mutex_unlock(&compass->mutex);
		compassStartMeasurement(compass);
		pthread_mutex_lock(&compass->mutex);
		/* 計測データ取得 */
		result = i2cRead(compass->device,response,sizeof(response));
		if(result < (int)sizeof(response)){
			ERROR_PRINT("i2cRead: failure\n");
			compass->angle = COMPASS_RESULT_NO_DATA;
		}else{
			compass->angle = (((uint16_t)response[0]) << 8) + response[1];
			/* 更新時刻を記録 */
			gettimeofday(&compass->lastUpdateTime, NULL);
		}
	}

	/* 計測データの単位を変換 */
	if(compass->angle < 3600){
		angle = compass->angle / 10.;
	}else{
		angle = nan("");
	}

	/* コンパスアンロック */
	pthread_mutex_unlock(&compass->mutex);

	return angle;
}

/*
 * コンパスの校正を開始する
 * 引数: コンパスポインタ
 * 戻値: 正常時 0
 *       異常時 -1
 */
int compassStartCalibration(Compass* compass){
	uint8_t command[1];
	int result;
	/* 引数確認 */
	if(compass == NULL){
		ERROR_PRINT("Compass is NULL\n");
		return -1;
	}
	/* コンパスロック */
	pthread_mutex_lock(&compass->mutex);
	/* 計測開始 */
	command[0] = COMMAND_START_CALIBRATION;
	result = i2cWrite(compass->device,command,sizeof(command));
	if(result < (int)sizeof(command)){
		ERROR_PRINT("i2cWrite: failure\n");
		result = -1;
	}else{
		result = 0;
	}
	/* コンパスアンロック */
	pthread_mutex_unlock(&compass->mutex);
	return result;
}

/*
 * コンパスの校正を終了する
 * 引数: コンパスポインタ
 * 戻値: 正常時 0
 *       異常時 -1
 */
int compassStopCalibration(Compass* compass){
	uint8_t command[1];
	int result;
	/* 引数確認 */
	if(compass == NULL){
		ERROR_PRINT("Compass is NULL\n");
		return -1;
	}
	/* コンパスロック */
	pthread_mutex_lock(&compass->mutex);
	/* 計測開始 */
	command[0] = COMMAND_STOP_CALIBRATION;
	result = i2cWrite(compass->device,command,sizeof(command));
	if(result < (int)sizeof(command)){
		ERROR_PRINT("i2cWrite: failure\n");
		result = -1;
	}else{
		result = 0;
	}
	/* コンパスアンロック */
	pthread_mutex_unlock(&compass->mutex);
	return result;
}
