/*
 * i2c.h
 *
 *  Created on: 2013/12/21
 *      Author: d10130
 *
 * I2Cマスターボードとの通信に必要な基本的な操作をまとめました。
 * スレッドセーフです。
 */

#ifndef I2C_H_
#define I2C_H_


#include <termios.h>
#include <stdint.h>

/*-------- システム情報 --------*/
/* 通信速度 (変更する場合はPIC側のプログラムの修正が必要) */
#define I2C_BAUD_RATE B230400
/* アドレスの最大値 */
#define I2C_ADDRESS_MAX 0x7F


/*-------- I2C通信時のエラーコード --------*/
/* 引数が間違っている */
#define I2C_INVAL -1
/* CPUボードとI2Cマスターボードの間で問題がある */
#define I2C_BUS_ERROR -2
/* I2CマスターボードとI2Cデバイスの間で問題がある */
#define I2C_DEVICE_ERROR -3

/*-------- I2Cマスターボードのデバイスファイル名の最大長さ --------*/
#define I2C_DEVICE_FILE_NAME_MAX 256


/*-------- I2C通信用構造体 --------*/
struct I2CBus_;
struct I2CDevice_;
/* I2Cバス構造体 */
typedef struct I2CBus_ I2CBus;
/* I2Cデバイス構造体 */
typedef struct I2CDevice_ I2CDevice;



/*
 * バスのオープン (I2Cマスターボードとの通信開始)
 * 引数: I2Cマスターボードのデバイスファイル
 * 戻値: 正常時 I2Cバスポインタ
 *       異常時 エラーが発生した場合はNULL (errnoで判断)
 */
I2CBus* i2cBusOpen(char* fileName);

/*
 * バスのクローズ (I2Cマスターボードとの通信終了)
 * 引数: 通信終了するI2Cバスポインタ
 */
void i2cBusClose(I2CBus* bus);

/*
 * バスのリセット (ブレーク信号送信)
 * 戻値: 正常時 0
 *       異常時 負の数 (最上部の一覧を参照)
 */
int i2cBusReset(I2CBus* bus);

/*
 * デバイスが存在するか判定
 * 引数: デバイスが接続されているバス, デバイスのアドレス
 * 戻値: 存在する   1
 *       存在しない 0
 */
int i2cBusHasDevice(I2CBus* bus, uint8_t address);

/*
 * デバイスのオープン
 * 引数: デバイスが接続されているバス, デバイスのアドレス
 * 戻値: 正常時 I2Cデバイスポインタ
 *       異常時 NULL (errnoで判断)
 */
I2CDevice* i2cDeviceOpen(I2CBus* bus, uint8_t address);

/*
 * デバイスのクローズ
 * 引数: 通信終了するI2Cデバイスポインタ
 */
void i2cDeviceClose(I2CDevice* device);

/*
 * デバイスにデータ送信
 * 引数: I2Cデバイスポインタ, 書き込むデータ, データサイズ
 * 戻値: 正常時 送信できたバイト数
 *       異常時 負の数 (最上部の一覧を参照)
 */
int i2cWrite(I2CDevice* device, void* buf, uint8_t size);

/*
 * デバイスからデータ受信
 * 引数: I2Cデバイスポインタ, 受信バッファ, データサイズ
 * 戻値: 正常時 受信できたバイト数
 *       異常時 負の数 (最上部の一覧を参照)
 */
int i2cRead(I2CDevice* device, void* buf, uint8_t size);


#endif /* I2C_H_ */
