/*
 * motor.c
 *
 *  Created on: 2013/07/27
 *      Author: d10130
 */
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include "motor.h"



// モーター型
struct motor_t {
	int fd;               // ファイルディスクリプタ
	int8_t is_active;     // stop状態か否か
	int8_t speed;         // 最後に設定した目標速度(現在の速度ではない)
	int8_t kp;            // 比例ゲイン
	int8_t ki;            // 積分ゲイン
	int32_t count_init;   // ロータリーエンコーダ初期値
};



// モーターとの通信を開始してモーターポインタを初期化する。
// 戻り値 成功:0 失敗:-1
int motor_open(const char* dev_file,Motor** p_motor){
	Motor* motor;

	motor = (Motor*)calloc(sizeof(Motor),1);
	if(motor == NULL){
		return -1;
	}

	motor->fd = open(dev_file,O_RDWR);
	if(motor->fd < 0){
		free(motor);
		return -1;
	}

	motor_set_kp(motor,MOTOR_DEFAULT_KP);
	motor_set_ki(motor,MOTOR_DEFAULT_KI);
	motor_set_count(motor,0);
	motor_set_speed(motor,0);
	motor_brake(motor);

	*p_motor = motor;
	return 0;
}



// モーターとの通信を終了する。
void motor_close(Motor* motor){
	motor_set_kp(motor,0);
	motor_set_ki(motor,0);
	motor_set_speed(motor,0);
	motor_brake(motor);

	close(motor->fd);
	free(motor);
}



// ロータリーエンコーダの値[pulse]を初期化
void motor_set_count(Motor* motor, int32_t count_init){
	motor_send_clear_packet(motor->fd);
	motor->count_init = count_init;
}



// ロータリーエンコーダの値[pulse]を取得
int32_t motor_get_count(Motor* motor){
	int8_t speed;
	uint16_t pwm;
	int32_t count;
	motor_recv_status_packet(motor->fd, &speed, &pwm, &count);

	count += motor->count_init;
	return count;
}



// モーターの回転速度[pulse/s]を変更
// -128~127[pulse/制御周期]を超えた場合は切り捨て
void motor_set_speed(Motor* motor, float pulse_par_sec){
	int8_t speed;
	float pulse_par_period;

	// [pulse/制御周期] <-- [pulse/s]
	pulse_par_period = pulse_par_sec * 1000.0 / MOTOR_DEFAULT_PERIOD;
	if(pulse_par_period < -128){
		speed = -128;
	}else if(pulse_par_period > 127){
		speed = 127;
	}else{
		speed = (int8_t)(pulse_par_period + 0.5); // 四捨五入
	}

	// MCBに送信
	motor_send_control_packet(motor->fd, speed, motor->kp, motor->ki);
	motor->speed = speed;

	// 停止していた場合はスタート信号送信
	if( motor->is_active == 0){
		motor_send_start_packet(motor->fd);
		motor->is_active = 1;
	}
}



// モーターの回転速度[pulse/s]を取得
float motor_get_speed(Motor* motor){
	int8_t speed;
	uint16_t pwm;
	int32_t count;

	// MCBにデータを要求
	motor_recv_status_packet(motor->fd, &speed, &pwm, &count);
	// [pulse/制御周期] --> [pulse/s]
	return ( speed * MOTOR_DEFAULT_PERIOD / 1000.0 );
}



// ブレーキを掛ける
void motor_brake(Motor* motor){
	motor_send_stop_packet(motor->fd);
	motor->is_active = 0;
}



// Kpの変更
void motor_set_kp(Motor* motor,uint8_t kp){
	motor_send_control_packet(motor->fd, motor->speed, kp, motor->ki);
	motor->kp = kp;
}



// Kiの変更
void motor_set_ki(Motor* motor,uint8_t ki){
	motor_send_control_packet(motor->fd, motor->speed, motor->kp, ki);
	motor->ki = ki;
}



// Kpの取得
uint8_t motor_get_kp(Motor* motor){
	return motor->ki;
}



// Kiの取得
uint8_t motor_get_ki(Motor* motor){
	return motor->kp;
}
