/*
 * run.h
 *
 *  Created on: 2013/07/28
 *      Author: d10130
 *
 */

//デバッグモードを使用しないときは下の行をコメントアウト
#define __debug_run_

#ifndef RUN_H_
#define RUN_H_

#include "motor.h"


// 走行情報構造体
// 制御方法によって使う情報が違うので、中身は個々のモジュールが独自に定義している。
// だからrun_straight_init()で生成したやつをrun_circle()に渡したりしないように。
struct run_status_t;
typedef struct run_status_t RunStatus;


/******** 停止 ********/
#define run_stop(motor_l,motor_r) do{motor_brake(motor_l);motor_brake(motor_r);}while(0)


/******** 直進 ********/

// 使用するモーター,初期位置の設定
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
extern RunStatus* run_straight_init(Motor* m_l,Motor* m_r);
// 繰り返し呼ぶと直進。目標速度 v[mm/s]。戻り値:走行距離[mm]
extern float run_straight(float v,double angle_ref,RunStatus* status);
// 繰り返し呼ぶと直進。目標距離 x[mm],上限速度 max_speed[mm/s]。戻り値:走行距離[mm]
extern float run_straight_to(float x,float max_speed,double angle_ref,RunStatus* status);
// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
extern void run_straight_finish(RunStatus* status);



/******** 旋回 ********/

// 使用するモーター,初期位置,旋回半径 r[mm]の設定
// r が正で反時計回り, r が負で時計回り
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
extern RunStatus* run_circle_init(Motor* m_l,Motor* m_r,float r);
// 繰り返し呼ぶと旋回。目標速度 v[mm/s]。戻り値:旋回角度[deg]
extern float run_circle(float v,RunStatus* status);
// 繰り返し呼ぶと旋回。目標旋回角度 angle[deg],上限速度 max_speed[mm/s]。戻り値:旋回角度[deg]
extern float run_circle_to(float angle,float max_speed,RunStatus* status);
// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
extern void run_circle_finish(RunStatus* status);



/******** 回転 ********/

// 使用するモーター,初期位置の設定
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
extern RunStatus* run_rotate_init(Motor* m_l,Motor* m_r);
// 繰り返し呼ぶと回転。目標角速度 angle_v[deg/s]。戻り値:回転角度[deg]
extern float run_rotate(float angle_v,RunStatus* status);
// 繰り返し呼ぶと回転。目標回転角度 angle[deg], 上限角速度 max_angle_speed[deg/s]。戻り値:回転角度[deg]
extern float run_rotate_to(float angle,float max_angle_speed,RunStatus* status);
// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
extern void run_rotate_finish(RunStatus* status);



/******** 壁沿いに進む ********/

// 使用するモーター,USSの設定
// uss_get(uss_l) < uss_get(uss_r) のとき反時計方向に旋回
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
extern RunStatus* run_trace_wall_init(Motor* m_l,Motor* m_r,uint8_t uss_l,uint8_t uss_r);
// 繰り返し呼ぶと壁沿いに進む。目標速度 v[mm/s]。戻り値:走行距離[mm]
// 正面に壁があっても回避できないので呼び出し側で対応してください
extern float run_trace_wall(float v,RunStatus* status);
// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
extern void run_trace_wall_finish(RunStatus* status);



#define SPD 15.0		/*モータースピードの設定.正のとき前進*/
#define	DIS_MARGIN 7.0	/*前進距離の許容誤差[mm]*/
#define DIR_MARGIN 1.5	/*回転角度の許容誤差[deg]*/


//絶対座標(x,yどちらか一方)と進行方向の角度で終点を指定して、その方向を向いて直進をする関数。
//引数  sel:x=[0],y=[1] 、reference_coor:目標絶対座標[mm]、dir:偏角[deg]
extern void run_absolute(int sel,float reference_coor,float dir);
//巡回とか既定ルート警備に用いる。相対座標で進める関数は別。
//タッチセンサで補正、怪盗機追跡とかは相対座標で
	


#endif /* RUN_H_ */
