/*
 * run_circle.c
 *
 *  Created on: 2013/07/28
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include "mirs.h"
#include "motor.h"
#include "run.h"

// 速度補正用ゲイン
#define V_KP 0.000001
#define V_KI 0.0
#define V_KD 0.0
// 位置補正用ゲイン
#define X_KP 0.001
#define X_KI 0.0001
#define X_KD 0.0

struct run_status_t {
	// 使用するモーター
	Motor* motor_l;
	Motor* motor_r;
	// 左右のエンコーダーの初期値
	int32_t count_l_init;
	int32_t count_r_init;
	// 速度補正用
	int32_t v_err;
	int32_t v_err_sum;
	int32_t v_err_last;
	// 位置補正用
	float x_err;
	float x_err_sum;
	float x_err_last;
	 // 回転半径
	float radius;
};



// 使用するモーター,初期位置,旋回半径 r[mm]の設定
// r が正で反時計回り, r が負で時計回り
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
RunStatus* run_circle_init(Motor* left,Motor* right,float r){
	RunStatus* st;
	st = (RunStatus*)malloc(sizeof(RunStatus));
	if(st == NULL){
		return NULL;
	}

	st->motor_l = left;
	st->motor_r = right;

	st->count_l_init = motor_get_count(st->motor_l);
	st->count_r_init = motor_get_count(st->motor_r);

	st->v_err = 0;
	st->v_err_sum = 0;
	st->v_err_last = 0;

	st->x_err = 0;
	st->x_err_sum = 0;
	st->x_err_last = 0;

	st->radius = r;

	return st;
}



// 繰り返し呼ぶと旋回。目標速度 v[mm/s]。戻り値:旋回角度[deg]
float run_circle(float v,RunStatus* st){
	int32_t count_l,count_r; // [pulse]
	float dv; // [pulse/s]

	v /= MIRS_DISTANCE_TO_PULSE_RATIO; // [mm/s] --> [pulse/s]

	count_l =  motor_get_count(st->motor_l) - st->count_l_init;
	count_r =  motor_get_count(st->motor_r) - st->count_r_init;

	st->v_err_last = st->v_err;
	st->v_err = (st->radius - MIRS_BODY_RADIUS)*count_r
			- (st->radius + MIRS_BODY_RADIUS)*count_l;
	st->v_err_sum += st->v_err;

	dv = v * MIRS_BODY_RADIUS / st->radius;
	dv += V_KP * st->v_err;
	dv += V_KI * st->v_err_sum;
	dv += V_KD * (st->v_err - st->v_err_last);

	motor_set_speed(st->motor_l,v+dv);
	motor_set_speed(st->motor_r,v-dv);

	return MIRS_DISTANCE_TO_PULSE_RATIO * (count_l + count_r)/2.0 / st->radius * 180.0 / M_PI;
}



// 繰り返し呼ぶと旋回。目標旋回角度 angle[deg],上限速度 max_speed[mm/s]。戻り値:旋回角度[deg]
float run_circle_to(float angle,float max_speed,RunStatus* st){
	float angle_now;
	float v;

	max_speed = fabs(max_speed);

	v = 0.0;
	v += X_KP * st->x_err;
	v += X_KI * st->x_err_sum;
	v += X_KD * (st->x_err - st->x_err_last);

	if( fabs(v) > max_speed ){
		v = (v > 0)?(max_speed):(-max_speed);
	}

	angle_now = run_circle(v,st);

	st->x_err_last = st->x_err;
	st->x_err = angle - angle_now;
	if( fabs(v) < max_speed ){
		st->x_err_sum += st->x_err;
	}
	return angle_now;
}


// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
void run_circle_finish(RunStatus* st){
	motor_brake(st->motor_l);
	motor_brake(st->motor_r);
	free(st);
}
