/*
 * run_straight.c
 *
 *  Created on: 2013/07/28
 *      Author: d10130
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <math.h>
#include "mirs.h"
#include "motor.h"
#include "run.h"
#include "compass.h"

// 速度補正用ゲイン
#define V_KP 0.06
#define V_KI 0.00
#define V_KD 0.3
// 位置補正用ゲイン
#define X_KP 0.005
#define X_KI 0.0003
#define X_KD 0.0

struct run_status_t {
	// 使用するモーター
	Motor* motor_l;
	Motor* motor_r;
	// 左右のエンコーダーの初期値
	int32_t count_l_init;
	int32_t count_r_init;
	// 速度補正用
	double v_err;
	double v_err_sum;
	double v_err_last;
	// 走行距離補正用
	double x_err;
	double x_err_sum;
	double x_err_last;
};



// 使用するモーター,初期位置の設定
// 走行情報を保存するための変数を作成して返す。失敗したらNULLを返す。
RunStatus* run_straight_init(Motor* m_l,Motor* m_r){
	RunStatus* st;
	st = (RunStatus*)malloc(sizeof(RunStatus));
	if(st == NULL){
		return NULL;
	}

	st->motor_l = m_l;
	st->motor_r = m_r;

	st->count_l_init = motor_get_count(st->motor_l);
	st->count_r_init = motor_get_count(st->motor_r);

	st->v_err = 0;
	st->v_err_sum = 0;
	st->v_err_last = 0;

	st->x_err = 0;
	st->x_err_sum = 0;
	st->x_err_last = 0;

	return st;
}



// 繰り返し呼ぶと直進。目標速度 v[mm/s]。目標角度angle_ref[deg]。戻り値:走行距離[mm]
float run_straight(float v,double angle_ref,RunStatus* st){
	int32_t count_l,count_r; // [pulse]
	float dv; // [pulse/s]

	v /= MIRS_DISTANCE_TO_PULSE_RATIO; // [mm/s] --> [pulse/s]

	count_l = motor_get_count(st->motor_l) - st->count_l_init;
	count_r = motor_get_count(st->motor_r) - st->count_r_init;

	st->v_err_last = st->v_err;
	if(angle_ref - angle > 180){		//0-360の壁を越えるかどうかで回転方向を修正
		st->v_err = angle_ref - angle - 360;	
	}else if(angle_ref - angle < -180){
		st->v_err = angle_ref - angle + 360;
	}else{
		st->v_err = angle_ref - angle;
	}
	st->v_err_sum += st->v_err;

	#ifdef __debug_run__
	printf("angle = %6.2f[deg]\t",angle);
	printf("angle_ref = %6.2f[deg]\n",angle_ref);
	printf("st->v_err = %lf\n",st->v_err);
	printf("st->v_err_sum = %lf\n",st->v_err_sum);
	printf("st->v_err_D = %lf\n",st->v_err - st->v_err_last);
	#endif

	dv = 0.0;
	dv += V_KP * st->v_err;
	dv += V_KI * st->v_err_sum;
	dv += V_KD * (st->v_err - st->v_err_last);

	#ifdef __debug_run__
	printf("l_speed = %f\t",v+dv);
	printf("r_speed = %f\n",v-dv);
	#endif

	motor_set_speed(st->motor_l,v+dv);
	motor_set_speed(st->motor_r,v-dv);

	return MIRS_DISTANCE_TO_PULSE_RATIO * (count_l + count_r)/2.0;
}



// 繰り返し呼ぶと直進。目標距離 x[mm],上限速度 max_speed[mm/s]。戻り値:走行距離[mm]
float run_straight_to(float x,float max_speed,double angle_ref,RunStatus* st){
	float x_now;
	float v;

	max_speed = fabs(max_speed);

	v = 0.0;
	v += X_KP * st->x_err;
	v += X_KI * st->x_err_sum;
	v += X_KD * (st->x_err - st->x_err_last);

	if( fabs(v) > max_speed ){
		v = (v > 0)?(max_speed):(-max_speed);
	}

	x_now = run_straight(v,angle_ref,st);
	
	st->x_err_last = st->x_err;
	st->x_err = x - x_now;
	st->x_err_sum += st->x_err;
	if( fabs(st->x_err) < fabs(st->x_err_last) ){
		st->x_err_sum = 0;
	}

	return x_now;
}



// ブレーキかけて free(status) する。呼び忘れと二重解放に注意。
void run_straight_finish(RunStatus* st){
	motor_brake(st->motor_l);
	motor_brake(st->motor_r);
	free(st);
}
